function plugindef()
	-- This function and the 'finaleplugin' namespace
	-- are both reserved for the plug-in definition.
	finaleplugin.MinJWLuaVersion = "0.54"
	finaleplugin.Author = "Chris"
	finaleplugin.Version = "2.0"
	finaleplugin.Date = "22/11/2017"
	finaleplugin.AuthorEmail = "chris@finale-logiciel-aide-gravure-musicale.eu"
	finaleplugin.CategoryTags = "Layout, System, UI"
	return "Copie de mise en page", "Copie de mise en page", ""
end

--[[
variables  globales
]]
local Paras = {}


--[[
FONCTIONS UTILITAIRES
]]
--[[
crée une FCString à partir d'une chaine Lua
]]
local function createString(texte)
	local str=finale.FCString()
	str.LuaString = texte
	return str
end

--[[
convertit une Table indexée en FCStrings
]]
local function tableToStrings(table)
	local strings =finale.FCStrings()
	for _, v in ipairs(table) do
		strings:AddCopy(createString(v))
	end
	return strings
end

--[[
    retourne le nom du fichier sans le path (Lua String)
    (le path d'un nouveau document contient uniquement le nom du fichier)
]]
local function getFilename(path)
	local filename = finale.FCString()
	if path:SplitToPathAndFile(NULL, filename) then
		return filename.LuaString
	else
		return path.LuaString
	end
end

--[[
    retourne la liste des propriétés d'un objet
]]
local function getProperties(object)
	local PROPPAGE = {"BottomMargin","Height","HoldMargins","LeftMargin","Percent","RightMargin","TopMargin","Width"}
	local PROPSTAFFSYSTEM = {"BottomMargin","HoldMargins","LeftMargin","Resize","ResizeVerticalSpace",
		"RightMargin","SpaceAbove","SpaceAfterMusic","SpaceBeforeMusic","StaffHeight","TopMargin","UseStaffResize"}
	local PROPSYSTEMSTAFF = {"Distance","Resize"}
	local PROPPAGETEXT = {"FirstPage","FrameEdgeRef","HorizontalAlignment","HorizontalAlignmentRightPage","HorizontalPos","HorizontalPosRightPage",
		"IndependentRightPage","LastPage","PageAssignment","PageEdgeRef","VerticalAlignment","VerticalPos","VerticalPosRightPage","Visible"}
	local PROPTEXTBLOCK = {"ExpandSingleWord","FrameReference","Height","HorizontalTextOffset","Justification","LineInset",
		"LineSpacing","LineSpacingIsPercent","LineWidth","ShapeID","ShowBorders","VerticalTextOffset","Width","WordWrap"}
	local PROPMULTIMEASUREREST = {"EndMeasure","NumberHorizontalAdjust","NumberVerticalAdjust","ShapeEndAdjust","ShapeID","ShapeStartAdjust",
		"StartMeasure","StartNumberingAt","SymbolSpace","UseSymbols","UseSymbolsLessThan","Width"}

	local propertiesNamesObjects = {FCPage = PROPPAGE, FCStaffSystem = PROPSTAFFSYSTEM, FCSystemStaff = PROPSYSTEMSTAFF,
		FCPageText = PROPPAGETEXT, FCTextBlock = PROPTEXTBLOCK, FCMultiMeasureRest = PROPMULTIMEASUREREST}
	local propertiesNames = propertiesNamesObjects[object:ClassName()]
	if not propertiesNames then return end

	local list = {}

	for _, v in pairs(propertiesNames) do
		list[v] = object[v]
	end

	return list
end

--[[
    écrit les propriétés d'un objet
]]
local function setObjectProperties(object, listProperties)
	for k, v in pairs(listProperties) do
		object[k] = v
	end
end

--[[
  écrit une info de pagebreak à la 1° mesure d'un système
]]
local function setPagebreak(system, pagebreak)
	local m = finale.FCMeasure()
	if m:Load(system.FirstMeasure) then
		m.PageBreak = pagebreak
		m:Save()
	end
end

--[[
  retourne un ou plusieurs objets FCMultiMeasureRest contenu dans la région d'un système (ou {})
]]
local function getMultiMeasureRest(musicregion)
	local ret = {}

	local multimeasurerests = finale.FCMultiMeasureRests()
	multimeasurerests:LoadAll()
	for mmr in each(multimeasurerests) do
		-- teste si la 1° mesure du multirest est dans le système (région)
		if musicregion:IsMeasureIncluded(mmr.StartMeasure) then
			table.insert(ret,mmr)
		end
	end

	return ret
end

--[[
  teste si la région ne contient aucune entrée
]]
local function isEntryEmptyRegion(musicregion)
	for m, s in eachcell(musicregion) do
		local noteentrycell = finale.FCNoteEntryCell(m, s)
		noteentrycell:Load()
		if not noteentrycell:IsEmpty() then return false end
	end
	return true
end
--[[
FIN FONCTIONS UTILITAIRES
]]


--[[
  OBJET PAGETEXT
]]
local objPageText = {} 
objPageText.__index = objPageText 

function objPageText:new(pagetext)
	self = setmetatable({}, objPageText)

	self.properties = getProperties(pagetext)
	local textBlock = pagetext:CreateTextBlock()
	self.propsTextBlock = getProperties(textBlock)
	self.string = textBlock:CreateRawTextString()
	self.id = {StringID = pagetext.StringID, TextBlockID = pagetext.TextBlockID}		-- pour pouvoir tester si le texte existe déjà dans la part (en cas de copie dans le même document)
	return self
end

-- Propriété  "properties"
function objPageText:getProperties()
	return self.properties
end

-- Propriété  du TextBlock
function objPageText:getPropsTextBlock()
	return self.propsTextBlock
end

-- Propriété  string
function objPageText:getString()
	return self.string
end

-- Propriété  ID
function objPageText:getID()
	return self.id
end
--[[
FIN OBJET PAGETEXT
]]



--[[
OBJET SYSTEMSTAVES
]]
local objSystemStaves = {} 
objSystemStaves.__index = objSystemStaves 

function objSystemStaves:new(ItemNo)
	self = setmetatable({}, objSystemStaves)

	local sysstaves = finale.FCSystemStaves()
	sysstaves:LoadAllForItem(ItemNo)
	self.listSysStaves = {}

	for sysstaff in each(sysstaves) do
		table.insert(self.listSysStaves,getProperties(sysstaff))
	end

	return self
end

-- retourne un objet "systemstaff"
function objSystemStaves:getItemAt(index)
	return self.listSysStaves[index]
end

-- écrire infos de portée
function objSystemStaves:setProperties(system)
	local sysstaves = finale.FCSystemStaves()
	sysstaves:LoadAllForItem(system.ItemNo)

	local ind = 1
	for sysstaff in each(sysstaves) do
		local propertiesSysStaff = self:getItemAt(ind)
		if propertiesSysStaff then
			setObjectProperties(sysstaff, propertiesSysStaff)
			sysstaff:Save()
		end
		ind = ind + 1
	end
end

--[[
FIN OBJET SYSTEMSTAVES
]]





--[[
OBJET SYSTEM
]]
local objSystem = {} 
objSystem.__index = objSystem 

function objSystem:new(indSystem)
	self = setmetatable({}, objSystem)

	finale.FCStaffSystems.UpdateFullLayout()  -- updater pour calculregion

	local system = finale.FCStaffSystem()
	if not system:Load(indSystem) then return end

	self.properties = getProperties(system)

	-- calculer le nombre de mesures du système
	local musicregion = finale.FCMusicRegion()
	system:CalcRegion(musicregion)
	self.nMeasures = musicregion:CalcMeasureSpan()

	-- mémoriser pagebreak
	self.pagebreak = system:HasPageBreak()

	-- Infos portées
	self.systemStaves = objSystemStaves:new(system.ItemNo)

	-- mémoriser MultiMeasureRests
	self.multiMeasureRests = {}
	for _, mmr in ipairs(getMultiMeasureRest(musicregion)) do
		table.insert(self.multiMeasureRests, {properties = getProperties(mmr), offsetMeasure = mmr.StartMeasure - musicregion.StartMeasure})
	end

	return self
end

-- Propriété  "properties"
function objSystem:get_Properties()
	return self.properties
end

-- Propriété  "nMeasures" (nombre de mesures du système)
function objSystem:getNMeasures()
	return self.nMeasures
end

-- Propriété  "pagebreak" (nombre de mesures du système)
function objSystem:getPagebreak()
	return self.pagebreak
end

-- Propriété  "systemStaves" 
function objSystem:getSystemStaves()
	return self.systemStaves
end

-- Propriété  "multiMeasureRests" 
function objSystem:getMultiMeasureRests()
	return self.multiMeasureRests
end

-- copie ce système dans un système du document destination
function objSystem:setSystem(indSystem, isFirstSystem)
	finale.FCStaffSystems.UpdateFullLayout()  -- nécessaire pour freeze
	local ss = finale.FCStaffSystem()
	if not ss:Load(indSystem) then return false end

	setObjectProperties(ss, self:get_Properties())
	ss:Save()

	if not Paras.KeepMeasures then
		local fc=ss:CreateFreezeSystem(true)
		fc.NextSysMeasure = ss.FirstMeasure + self:getNMeasures()
		fc:Save()
	end

	if isFirstSystem and indSystem ~= 1 then
		-- mettre un pagebreak pour le 1° système  d'une page autre que la 1°
		setPagebreak(ss, true)
	else
		-- pagebreak (toujours écrire / pour éventuellement supprimer)
		setPagebreak(ss, self:getPagebreak())
	end

	-- écrire infos de portée
	local objSystemStaves = self:getSystemStaves()
	objSystemStaves:setProperties(ss)

	-- créer MultiMeasureRests
	if Paras.MultiMeasureRest then
		for _, mmrInfos in ipairs(self:getMultiMeasureRests()) do
			local mmrDest = finale.FCMultiMeasureRest()
			setObjectProperties(mmrDest, mmrInfos.properties)
			local span = mmrDest.EndMeasure - mmrDest.StartMeasure
			mmrDest.StartMeasure = ss.FirstMeasure + mmrInfos.offsetMeasure
			mmrDest.EndMeasure = mmrDest.StartMeasure + span

			-- tester si la région est vide
			-- tester si un MultiMeasureRest existe et le supprimer
			local musicregion = finale.FCMusicRegion()
			musicregion:SetFullDocument()	-- initialisation
			musicregion.StartMeasure = mmrDest.StartMeasure
			musicregion.EndMeasure = mmrDest.EndMeasure

			for _, mmr in ipairs(getMultiMeasureRest(musicregion)) do
				mmr:DeepDeleteData()
			end

			if isEntryEmptyRegion(musicregion) then	mmrDest:Save() end
		end
	end

	return true
end

--[[
FIN OBJET SYSTEM
]]



--[[
OBJET PAGE
]]
local objPage = {} 
objPage.__index = objPage 

function objPage:new(numPage)
	self = setmetatable({}, objPage)

	self:getPageProperties(numPage)
	if not self:getFirstSystem() then return end

	self.systemes = {}
	for indSystem = self:getFirstSystem() , self:getLastSystem()  do
		table.insert(self.systemes ,objSystem:new(indSystem))
	end

	-- PageTexts
	self.pagetexts = {}
	local pagetexts = finale.FCPageTexts()
	pagetexts:LoadAllForItem(numPage)
	for pagetext in each(pagetexts) do
		table.insert(self.pagetexts ,objPageText:new(pagetext))
	end

	return self
end

-- Propriété  "properties" 
function objPage:getProperties()
	return self.properties
end

-- Propriété  "firstSystem" 
function objPage:getFirstSystem()
	return self.firstSystem
end

-- Propriété  "lastSystem" 
function objPage:getLastSystem()
	return self.lastSystem
end

-- Retourne un système
function objPage:getItemAt(index)
	return self.systemes[index]
end

-- Retourne tous les systèmes
function objPage:getSystems()
	return self.systemes
end

-- Retourne tous les PageTexts
function objPage:getPageTexts()
	return self.pagetexts
end

-- mémorise les propriétés de la page
function objPage:getPageProperties(numPage)
	local p = finale.FCPage()
	if not p:Load(numPage) then return nil end

	local first = p:GetFirstSystem()
	if first == 0 then
		p:UpdateLayout(false)
		first = p:GetFirstSystem()
	end

	local last = p:CalcLastSystem()
	if last == 0 then
		p:UpdateLayout(false)
		last = p:CalcLastSystem()
	end

	self.properties = getProperties(p)
	self.firstSystem = first
	self.lastSystem = last
end

-- copie les propriétés de cette page dans la page de destination (indPageDest)
function objPage:setPageProperties(indPageDest)
	local p = finale.FCPage()
	if not p:Load(indPageDest) then return end

	setObjectProperties(p, self:getProperties())       
	p:Save()
	p:UpdateLayout(false)
end

-- modifie toutes les propriétés des systèmes (avec les portées) dans le document destination
function objPage:setSystemsProperties(firstSystem)
	-- parcourir et éditer tous les systèmes
	local indSystem = firstSystem
	for _, objSystem in ipairs(self:getSystems()) do
		local isSetSystem = objSystem:setSystem(indSystem, indSystem == firstSystem)
		if not isSetSystem then return nil end
		indSystem = indSystem + 1
	end
	return indSystem
end

-- copie les pagetexts dans le document destination
function objPage:setPageTextsProperties(indPageDest)
	local newPageText = true	-- indique s'il faut créer un nouvel objet FCPageText
	local isVisibleOnlyPart = false	-- indique si le texte créé ne doit être visible que sur la part

	-- parcourir tous les pagetexts
	for _, pagetext in ipairs(self:getPageTexts()) do
		-- créer un objet FCPageText à partir de la sauvegarde de la source
		local pagetextDest = finale.FCPageText()
		local properties = pagetext:getProperties()
		setObjectProperties(pagetextDest, properties) 
		pagetextDest.FirstPage = indPageDest
		pagetextDest.LastPage = indPageDest + properties.LastPage - properties.FirstPage

		-- définir le type de copie
		if Paras.DocSourceID == Paras.DocDestID then -- copie à l'intérieur d'un document	
			if Paras.PartSourceItemNo == Paras.PartDestItemNo then -- copie à l'intérieur d'un document dans la même part
				if Paras.PartDestItemNo > 0 then -- copie à l'intérieur d'un document dans la même part (sauf score)
					isVisibleOnlyPart = true
				end

			else -- copie à l'intérieur d'un document entre 2 parts différentes
				local pageTextDestEqual = nil	-- le pagetext de la destination qui est identique à la source

				local allPageTextsDest = finale.FCPageTexts()
				allPageTextsDest:LoadAllForItem(indPageDest)
				for onePageTextDest in each(allPageTextsDest) do
					local objID = pagetext:getID()
					if objID.StringID == onePageTextDest.StringID and objID.TextBlockID == onePageTextDest.TextBlockID then
						pageTextDestEqual = onePageTextDest
						break
					end
				end

				if pageTextDestEqual then -- si le texte existe déjà
					-- changer uniquement les coordonnées
					pageTextDestEqual.HorizontalPos = pagetextDest.HorizontalPos
					pageTextDestEqual.VerticalPos = pagetextDest.VerticalPos
					pageTextDestEqual:Save()
					newPageText = false

				else -- le texte n'existe pas
					if Paras.PartDestItemNo > 0 then-- copie entre 2 documents différents vers une partie (entre 2 parts différentes)
						isVisibleOnlyPart = true
					end
				end
			end
		else	-- copie entre 2 documents différents
			if Paras.PartDestItemNo > 0 then -- copie entre 2 documents différents sur une partie (pas le score)
				isVisibleOnlyPart = true
			end
		end


		if newPageText then
			-- créer le textblock
			pagetextDest:SaveNewTextBlock(pagetext:getString())

			-- récupérer le textblock
			local textBlock = pagetextDest:CreateTextBlock()
			setObjectProperties(textBlock, pagetext:getPropsTextBlock()) 
			textBlock:Save()

			-- sauver le nouvel objet FCPageText
			pagetextDest:SaveNew(indPageDest)

			if isVisibleOnlyPart then
				local allparts = finale.FCParts()
				allparts:LoadAll()

				for part in each(allparts) do
					pagetextDest.Visible = part:IsCurrent()    
					part:SwitchTo()
					pagetextDest:Save()
					part:SwitchBack()    
				end
			end
		end
	end
end

-- copie tous les objets dans le document destination
function objPage:setAllPagesObjects(indPageDest, firstSystem)
	self:setPageProperties(indPageDest)
	local indFirstSystemNextPage = self:setSystemsProperties(firstSystem)

	if Paras.CopyTexts then	self:setPageTextsProperties(indPageDest) end

	return indFirstSystemNextPage
end
--[[
FIN OBJET PAGE
]]



--[[
OBJET PAGES
]]
local objPages = {} 
objPages.__index = objPages 

function objPages:new()
	self = setmetatable({}, objPages)

	self.pages = {}
	for numPage = Paras.FirstPageSource, Paras.LastPageSource do
		table.insert(self.pages, objPage:new(numPage))
	end

	return self
end

-- retourne une page
function objPages:getItemAt(numPage)
	return self.pages[numPage]
end

-- retourne le nombre de pages
function objPages:Count()
	return #self.pages
end

-- supprime une page
function objPages:DeletePage(index)
	table.remove(self.pages, index)
end

-- écrit toutes les données sur la destination
function objPages:setDatas(indSource)
	local objFirstPageDest = objPage:new(Paras.FirstPageDest)
	local firstSystemNextPageDest = objFirstPageDest:getFirstSystem()
	objFirstPageDest = nil -- libérer l'objet
	if not firstSystemNextPageDest then return end

	-- parcourir toutes les pages destination et définir les systèmes
	for indPage = Paras.FirstPageDest, Paras.LastPageDest do
		local objPage = self:getItemAt(indSource)

		if objPage then
			firstSystemNextPageDest = objPage:setAllPagesObjects(indPage, firstSystemNextPageDest)
			if not firstSystemNextPageDest then break end

			-- ajouter un pagebreak au système d'après (s'il existe)
			finale.FCStaffSystems.UpdateFullLayout()
			local ss = finale.FCStaffSystem()
			if  ss:Load(firstSystemNextPageDest) then
				setPagebreak(ss,true)
			end
		end

		indSource = indSource + 1
		indSource = ((indSource - 1) % self:Count()) + 1	-- si plus de pages en destination, on recommence à la 1° page source
	end

	finale.FCStaffSystems.UpdateFullLayout()
end

-- méthode : écrit toutes les données
function objPages:setPages()
	self:setDatas(1)

	-- tester si de nouvelles pages ont été crées dans la destination
	-- et si le nombre de pages de la dest est inférieur à celui de la source
	local flNewPages
	repeat
		flNewPages = false

		-- si la dernière page de copie de la destination est la dernière page de la partie
		-- voir si des nouvelles pages ont été créées
		if Paras.LastPageDest == Paras.PageDestCount then	
			local nPagesSource = Paras.LastPageSource - Paras.FirstPageSource + 1
			local nPagesDest = Paras.LastPageDest - Paras.FirstPageDest + 1
			local offsetPages = nPagesSource - nPagesDest

			local pages = finale.FCPages()
			pages:LoadAll()

			if pages.Count > Paras.PageDestCount then	-- une ou plusieurs pages ont été ajoutées à la destination
				if offsetPages > 0 then	-- la source comporte plus de pages que la destination
					-- supprimer les datas des pages source déjà traitées
					for _ = 1, nPagesDest do
						self:DeletePage(1)
					end
				end

				local indSource
				if offsetPages > 0 then
					indSource = offsetPages
				else
					indSource = (nPagesDest % nPagesSource) + 1
				end
				Paras.FirstPageDest = Paras.LastPageDest + 1
				Paras.LastPageDest = pages.Count
				Paras.PageDestCount = pages.Count
				self:setDatas(indSource)
				flNewPages = true
			end
		end
	until not flNewPages

	finenv.UI():RedrawDocument()
end

--[[
FIN OBJET PAGES
]]




--[[
OBJET PART
]]
local objPart = {} 
objPart.__index = objPart 

function objPart:new(part)
	self = setmetatable({}, objPart)

	self:setNamePart(part)
	self:setCountPages()
	self.itemNo = part.ItemNo

	return self
end

-- Propriété  "name" 
function objPart:getName()
	return self.name
end

-- Propriété  "itemNo" 
function objPart:getItemNo()
	return self.itemNo
end

-- Propriété  "countPages"
function objPart:getCountPages()
	return self.countPages
end

-- récupère le nom de la part
function objPart:setNamePart(part)
	local name = part:CreateCustomTextString()

	if not name then
		name = finale.FCString()
		part:GetName(name)
	end

	name:TrimEnigmaTags()

	if name:IsEmpty() then
		if part.ItemNo == 0 then
			name.LuaString = "Conducteur"
		else
			name.LuaString = "Partie " .. part.ItemNo
		end
	end

	self.name = name.LuaString
end

-- récupère le nombre de pages de la part
function objPart:setCountPages()
	local pages = finale.FCPages()
	pages:LoadAll()
	self.countPages = pages.Count
end


--retourne la liste des pages (FCStrings) d'une part
function objPart:getListPages()
	local list = finale.FCStrings()

	for n = 1 , self:getCountPages() do
		list:AddCopy(createString("Page " .. n))
	end

	return list
end

--[[
FIN OBJET PART
]]




--[[
OBJET DOCUMENT
]]
local objDoc = {} 
objDoc.__index = objDoc 

function objDoc:new(doc)
	self = setmetatable({}, objDoc)

	local path = finale.FCString()
	doc:GetPath(path)
	self.filename = getFilename(path)
	self.ID = doc.ID
	self.parts = {}

	local function callbackParts(part)
		if not part then return end
		table.insert(self.parts, objPart:new(part))
	end

	local iterator = finale.FCLuaIterator()
	iterator:ForEachPart(callbackParts)

	return self
end

-- Propriété  "filename" 
function objDoc:getFilename()
	return self.filename
end

-- Propriété  "ID"
function objDoc:getID()
	return self.ID
end

-- Propriété  "Parts"
function objDoc:getParts()
	return self.parts
end

-- Méthode
function objDoc:getItemAt(index)
	return self.parts[index]
end

-- Méthode : retourne la liste des noms des parts (FCStrings)
function objDoc:getPartNames()
	local list = finale.FCStrings()

	for _, part in ipairs(self:getParts()) do
		list:AddCopy(createString(part:getName()))
	end

	return list
end

--[[
FIN OBJET DOCUMENT
]]


--[[
OBJET DOCUMENTS (Collection)
]]
local objDocs = {} 
objDocs.__index = objDocs 

function objDocs:new()
	self = setmetatable({}, objDocs)

	self.docs = {}

	local function mydocumentcallback(doc)
		if not doc then return end
		table.insert(self.docs, objDoc:new(doc))
	end

	local iterator = finale.FCLuaIterator()
	iterator:ForEachDocument(mydocumentcallback)

	return self
end

-- Propriété  "docs" 
function objDocs:getDocs()
	return self.docs
end

-- Méthode
function objDocs:getItemAt(index)
	return self.docs[index]
end

-- retourne la liste des noms des fichiers
function objDocs:getListFilenameDocs()
	local list = {}

	for _, v in ipairs(self:getDocs()) do
		table.insert(list, v:getFilename())
	end

	return list
end

--[[
FIN OBJET DOCUMENTS
]]


--[[
CLASSE : BOITE DE DIALOGUE
]]
local DialogBoxClass = {} -- the table representing the class, which will double as the metatable for the instances
DialogBoxClass.__index = DialogBoxClass

-- ***** CONSTRUCTEUR *****
function DialogBoxClass:new(titre, datas, events)
	self = setmetatable({}, DialogBoxClass)

	self.dialogRef = finale.FCCustomLuaWindow()
	self.dialogRef:SetTitle(createString(titre))
	self.controlNames = {}

	-- création des controles
	for _, properties in ipairs(datas) do
		self:createControl(properties)
	end

	-- création des buttons OK et Cancel
	self.dialogRef:CreateOkButton()
	self.dialogRef:CreateCancelButton()

	-- événements
	for evt, callback in pairs(events) do
		self.dialogRef["Register" .. evt](self.dialogRef, callback) 
	end

	return self
end

-- ***** PROPRIETES *****

-- retourne la référence de la boite de dialogue
function DialogBoxClass:getDialogRef()
	return self.dialogRef
end

-- retourne la référence d'un controle
function DialogBoxClass:getCtrl(name)
	return self.controlNames[name]
end



-- ***** METHODES *****

-- affiche la boite de dialogue
function DialogBoxClass:display(parent)
	local BTOK = 1
	local ret = self:getDialogRef():ExecuteModal(parent)
	return ret == BTOK
end

function DialogBoxClass:initDataList(properties)
	local datalist = self:getCtrl(properties.name)
	if not datalist then return end
	local columns = properties.arguments.columns 
	if not columns then return end

	for _, column  in pairs(columns) do
		if type(column[1]) == "string" then column[1] = createString(column[1]) end
		datalist:AddColumn(column[1],column[2])
	end

	local rows = properties.arguments.rows 
	if not rows then return end

	for  _, row in ipairs(rows) do
		local dataListRow = datalist:CreateRow()
		dataListRow:ClearAll()	-- effacer la collection pour pouvoir rajouter des FCString()
		if type(row) ~= "table" then row = {row} end   -- 1 seule colonne

		for  _, item in ipairs(row) do	-- colonnes
			if type(item) ~= "userdata" then item = createString(item) end
			dataListRow:AddCopy(item)
		end
	end
end

function DialogBoxClass:createControl(properties)
	local method = self:getDialogRef()[properties.fn]
	if not method then -- ce n'est pas une méthode de FCCustomLuaWindow
		method = self[properties.fn]
		if method then -- c'est une méthode de DialogBoxClass
			method(self, properties)
		else -- ce n'est pas une méthode de FCCustomLuaWindow, ni de DialogBoxClass
			print(properties.fn .. " n'est pas une méthode de FCCustomLuaWindow, ni de DialogBoxClass")
		end
		return
	end

	-- la majorité des méthodes de création de controle, possède 2 arguments : left et top
	-- seules "CreateHorizontalLine" et "CreateVerticalLine" en ont 3
	-- le 3° argument est donc mis à à zéro et est redéfini après (top et left probablement)
	local control = method(self:getDialogRef(), 0, 0, 0)

	for prop, value  in pairs(properties) do
		if control[prop] then    -- si la méthode existe
			local arguments = {}

			if type(value) ~= "table" then  -- un seul argument
				value = {value} -- convertir en table
			end

			-- parcourir tous les arguments
			for _, argument in ipairs(value) do
				if type(argument) == "table" then -- convertir en FCStrings
					table.insert(arguments, tableToStrings(argument))

				elseif type(argument) == "string" then
					if string.sub(argument, 1, 9) == "pControl-" then	-- controle
						table.insert(arguments, self:getCtrl(string.sub(argument, 10)))
					else
						table.insert(arguments, createString(argument))
					end
				else	-- argument autre que table et string
					table.insert(arguments, argument)
				end
			end

			-- si appel avec plus d'arguments, pas de problème (mais moins si)
			-- maximum de 3 arguments
			control[prop](control, arguments[1], arguments[2], arguments[3])

		end
	end

	if properties.name then self.controlNames[properties.name] = control end  -- mémoriser le nom du controle s'il existe
end

--[[
FIN CLASSE : BOITE DE DIALOGUE
]]



--[[
obtient la liste des propriétés des pages sélectionnées et de leurs des systèmes (avec les portées) du document source
]]
local function getObjPages()
	return objPages:new()
end

local function setObjPages(objPages)
	objPages:setPages()
end

--[[
exécute une fonction sur un document
(maximum de 1 argument pour runFunction)
]]
local function docOperations(docs,IDDoc,ItemNoPart, runFunction, ...)
	local doc = docs:FindID(IDDoc)
	if not doc then return nil end
	local arg = {...}
	local ret

	local undotext = createString(plugindef() .. " [JW Lua]")
	local isCurrentDoc = doc:IsCurrent()
	if not isCurrentDoc then
		if not doc:SwitchTo(undotext, true) then return end
	end

	-- switcher sur la part
	local parts = finale.FCParts()
	parts:LoadAll()
	for part in each(parts) do      -- GetItemAt ne fonctionne pas (peut-être OrderID)
		if part.ItemNo == ItemNoPart then
			if part:SwitchTo() then
				-- exécuter la fonction
				ret = runFunction(arg[1])
				part:SwitchBack()
			end
			break
		end
	end

	if not isCurrentDoc then doc:SwitchBack(true) end

	return ret
end


--[[
crée et affiche une boite de dialogue
]]
local function createDialog()
	local listObjDocs = objDocs:new()
	local listFilenameDocs = listObjDocs:getListFilenameDocs()
	local comboPages = { nameStr = {"comboSourceFirstPage","comboSourceLastPage","comboDestFirstPage","comboDestLastPage"},
		name = {},
		selectedItem = {}}

	local datas = {
		{fn = "CreateStatic", SetLeft = 20, SetWidth = 200, SetText = "SOURCE"},
		{fn = "CreateStatic", SetLeft = 20, SetTop = 20, SetWidth = 200, SetText = "Document :"},
		{fn = "CreatePopup", name = "comboSource", SetLeft = 20, SetTop = 35, SetWidth = 300, SetStrings = {listFilenameDocs}, SetSelectedItem = 0},
		{fn = "CreateStatic", SetLeft = 20, SetTop = 60, SetWidth = 200, SetText = "Conducteur / Parties :"},
		{fn = "CreatePopup", name = "comboPartSource", SetLeft = 20, SetTop = 75, SetWidth = 300, SetSelectedItem = 0},
		{fn = "CreateButton", name = "btAllPagesSource", SetLeft = 180, SetTop = 125, SetWidth = 140, SetText = "Toutes les pages"},
		{fn = "CreateStatic", SetLeft = 20, SetTop = 135, SetWidth = 110, SetText = "Première Page :"},
		{fn = "CreatePopup", name = comboPages.nameStr[1], SetLeft = 20, SetTop = 150, SetWidth = 300, SetSelectedItem = 0},
		{fn = "CreateStatic", SetLeft = 20, SetTop = 175, SetWidth = 200, SetText = "Dernière Page :"},
		{fn = "CreatePopup", name = comboPages.nameStr[2], SetLeft = 20, SetTop = 190, SetWidth = 300, SetSelectedItem = 0},

		{fn = "CreateVerticalLine", SetLeft = 350, SetHeight = 280},

		{fn = "CreateStatic", SetLeft = 380, SetWidth = 200, SetText = "DESTINATION"},
		{fn = "CreateStatic", SetLeft = 380, SetTop = 20, SetWidth = 200, SetText = "Document :"},
		{fn = "CreatePopup", name = "comboDest", SetLeft = 380, SetTop = 35, SetWidth = 300, SetStrings = {listFilenameDocs}, SetSelectedItem = 0},
		{fn = "CreateStatic", SetLeft = 380, SetTop = 60, SetWidth = 200, SetText = "Conducteur / Parties :"},
		{fn = "CreatePopup", name = "comboPartDest", SetLeft = 380, SetTop = 75, SetWidth = 300, SetSelectedItem = 0},
		{fn = "CreateButton", name = "btAllPagesDest", SetLeft = 540, SetTop = 125, SetWidth = 140, SetText = "Toutes les pages"},
		{fn = "CreateStatic", SetLeft = 380, SetTop = 135, SetWidth = 110, SetText = "Première Page :"},
		{fn = "CreatePopup", name = comboPages.nameStr[3], SetLeft = 380, SetTop = 150, SetWidth = 300, SetSelectedItem = 0},
		{fn = "CreateStatic", SetLeft = 380, SetTop = 175, SetWidth = 200, SetText = "Dernière Page :"},
		{fn = "CreatePopup", name = comboPages.nameStr[4], SetLeft = 380, SetTop = 190, SetWidth = 300, SetSelectedItem = 0},

		{fn = "CreateCheckbox", name = "cbTexts", SetLeft = 20, SetTop = 240, SetWidth = 270, SetText = "Copier les textes"},
		{fn = "CreateCheckbox", name = "cbMultiMeasureRest", SetLeft = 380, SetTop = 240, SetWidth = 270, SetText = "Conserver les mesures à compter"},
		{fn = "CreateCheckbox", name = "cbKeepMeasures", SetLeft = 380, SetTop = 260, SetWidth = 300, SetText = "Conserver le nombre de mesures des systèmes"},

		{fn = "CreateHorizontalLine", SetLeft = 20, SetTop = 285, SetWidth = 660},

		{fn = "CreateStatic",  SetLeft = 20, SetTop = 320, SetText = "JW Lua " .. finenv.StringVersion}
	}

	local dialogClass
	local selectedItemSource, selectedItemDest, selectedItemPartSource, selectedItemPartDest
	local comboSource, comboDest, comboPartSource, comboPartDest
	local btAllPagesSource, btAllPagesDest

	-- événements
	local function OKButtonPressed()
		-- définir paramètres de retour
		local docSource = listObjDocs:getItemAt(comboSource:GetSelectedItem() + 1)
		local docDest = listObjDocs:getItemAt(comboDest:GetSelectedItem() + 1)
		local partSource = docSource:getItemAt(comboPartSource:GetSelectedItem() + 1)
		local partDest = docDest:getItemAt(comboPartDest:GetSelectedItem() + 1)

		Paras.DocSourceID = docSource:getID()
		Paras.PartSourceItemNo = partSource:getItemNo()
		Paras.FirstPageSource = comboPages.name[1]:GetSelectedItem() + 1
		Paras.LastPageSource = comboPages.name[2]:GetSelectedItem() + 1

		Paras.DocDestID = docDest:getID()
		Paras.PartDestItemNo = partDest:getItemNo()
		Paras.FirstPageDest = comboPages.name[3]:GetSelectedItem() + 1
		Paras.LastPageDest = comboPages.name[4]:GetSelectedItem() + 1
		Paras.PageDestCount = comboPages.name[3]:GetCount()	-- nombre de pages total de la destination

		Paras.CopyTexts = dialogClass:getCtrl("cbTexts"):GetCheck() == 1
		Paras.MultiMeasureRest = dialogClass:getCtrl("cbMultiMeasureRest"):GetCheck() == 1
		Paras.KeepMeasures = dialogClass:getCtrl("cbKeepMeasures"):GetCheck() == 1
	end

	local function callbackCommandChild(ctrl)
		local function setListComboPart(combo,comboOrigin)
			local docOrigin = listObjDocs:getItemAt(comboOrigin:GetSelectedItem() + 1)
			local listParts = docOrigin:getPartNames()
			combo:SetStrings(listParts)
			combo:SetSelectedItem(0)
		end

		local function setListPages(comboFirstPage,comboLastPage,comboOriginDoc,comboOriginPart)
			local docOrigin = listObjDocs:getItemAt(comboOriginDoc:GetSelectedItem() + 1)
			local partOrigin = docOrigin:getItemAt(comboOriginPart:GetSelectedItem() + 1)
			local list = partOrigin:getListPages()

			comboFirstPage:SetStrings(list)
			comboLastPage:SetStrings(list)
			comboFirstPage:SetSelectedItem(0)
			comboLastPage:SetSelectedItem(list.Count - 1)

			-- réinitialiser les valeurs mémorisées des comboPages
			for ind = 1,#comboPages.name do
				comboPages.selectedItem[ind] = comboPages.name[ind]:GetSelectedItem() 
			end
		end

		-- comboSource change
		if ctrl:GetControlID() == comboSource:GetControlID()  then
			setListComboPart(comboPartSource,comboSource)   -- changer la liste des parts
			setListPages(comboPages.name[1],comboPages.name[2],comboSource,comboPartSource)  -- changer la liste des pages

			-- comboDest change
		elseif ctrl:GetControlID() == comboDest:GetControlID() then
			setListComboPart(comboPartDest,comboDest)   -- changer la liste des parts
			setListPages(comboPages.name[3],comboPages.name[4],comboDest,comboPartDest)  -- changer la liste des pages

			-- comboPartSource change
		elseif ctrl:GetControlID() == comboPartSource:GetControlID() then
			setListPages(comboPages.name[1],comboPages.name[2],comboSource,comboPartSource)  -- changer la liste des pages

			-- comboPartDest change
		elseif ctrl:GetControlID() == comboPartDest:GetControlID() then
			setListPages(comboPages.name[3],comboPages.name[4],comboDest,comboPartDest)  -- changer la liste des pages

		end

		selectedItemSource = comboSource:GetSelectedItem()
		selectedItemDest = comboDest:GetSelectedItem()
		selectedItemPartSource = comboPartSource:GetSelectedItem()
		selectedItemPartDest = comboPartDest:GetSelectedItem()
	end


	local function callbackCommand(ctrl)

		local function isChanged()
			if comboSource:GetSelectedItem() ~= selectedItemSource then return true end
			if comboDest:GetSelectedItem() ~= selectedItemDest then return true end
			if comboPartSource:GetSelectedItem() ~= selectedItemPartSource then return true end
			if comboPartDest:GetSelectedItem() ~= selectedItemPartDest then return true end
			return false
		end

		-- parcourir les combos pages et tester la validité des numéros de page
		for ind, ctrlName in ipairs(comboPages.name) do
			if ctrl:GetControlID() == ctrlName:GetControlID() then
				if comboPages.name[2]:GetSelectedItem() < comboPages.name[1]:GetSelectedItem() or
				comboPages.name[4]:GetSelectedItem() < comboPages.name[3]:GetSelectedItem() then
					for i = 1,#comboPages.name do
						comboPages.name[i]:SetSelectedItem(comboPages.selectedItem[i])
					end
				end

				comboPages.selectedItem[ind] = comboPages.name[ind]:GetSelectedItem() 
				return
			end
		end

		-- clic bouton btAllPagesSource
		if ctrl:GetControlID() == btAllPagesSource:GetControlID() then
			comboPages.name[1]:SetSelectedItem(0)
			comboPages.name[2]:SetSelectedItem(comboPages.name[2]:GetCount() - 1)

			-- clic bouton btAllPagesDest
		elseif ctrl:GetControlID() == btAllPagesDest:GetControlID() then
			comboPages.name[3]:SetSelectedItem(0)
			comboPages.name[4]:SetSelectedItem(comboPages.name[4]:GetCount() - 1)

			-- tester si un para a changé (sauf les pages)
		elseif isChanged() then
			callbackCommandChild(ctrl)
		end
	end
	-- ***** FIN EVENEMENTS *****

	-- définir les événements à surveiller
	local events = {}
	events.HandleOkButtonPressed = OKButtonPressed
	events.HandleCommand = callbackCommand

	dialogClass = DialogBoxClass:new(plugindef() .. " - " .. finaleplugin.Version, datas, events)
	comboSource = dialogClass:getCtrl("comboSource")
	comboDest = dialogClass:getCtrl("comboDest")
	comboPartSource = dialogClass:getCtrl("comboPartSource")
	comboPartDest= dialogClass:getCtrl("comboPartDest")
	btAllPagesSource = dialogClass:getCtrl("btAllPagesSource")
	btAllPagesDest = dialogClass:getCtrl("btAllPagesDest")
	for ind, name in ipairs(comboPages.nameStr) do
		comboPages.name[ind] = dialogClass:getCtrl(name)
	end

	-- initialisation des 4 combos
	callbackCommandChild(comboSource)
	callbackCommandChild(comboDest)
	callbackCommandChild(comboPartSource)
	callbackCommandChild(comboPartDest)


	return dialogClass:display(nil)
end



--[[
fonction principale
]]
local function main()
	local isOK = createDialog()
	if not isOK then return end
	collectgarbage ("collect")  

	if Paras.DocSourceID == Paras.DocDestID and  Paras.PartSourceItemNo == Paras.PartDestItemNo and Paras.FirstPageSource == Paras.FirstPageDest then
		finenv.UI():AlertInfo("La source et la destination sont identiques. Aucune opération ne sera effectuée.",plugindef())
		return
	end

	local docs = finale.FCDocuments()
	docs:LoadAll()

	local objPages = docOperations(docs,Paras.DocSourceID, Paras.PartSourceItemNo, getObjPages)
	if not objPages then return false end
	docOperations(docs, Paras.DocDestID, Paras.PartDestItemNo, setObjPages, objPages)

end


--[[
DEBUT DU PROGRAMME
]]
main()